# Generated by roxygen2: do not edit by hand

S3method(summary,ssp.glm)
S3method(summary,ssp.glm.rF)
S3method(summary,ssp.quantreg)
S3method(summary,ssp.relogit)
S3method(summary,ssp.softmax)
export(ssp.glm)
export(ssp.glm.rF)
export(ssp.quantreg)
export(ssp.relogit)
export(ssp.softmax)
importFrom(Rcpp,evalCpp)
importFrom(quantreg,rq)
importFrom(stats,as.formula)
importFrom(stats,coef)
importFrom(stats,model.frame)
importFrom(stats,model.matrix)
importFrom(stats,model.response)
importFrom(stats,pnorm)
importFrom(stats,quantile)
importFrom(stats,quasibinomial)
importFrom(stats,runif)
useDynLib(subsampling)
